from django.contrib import admin
from django.urls import include
from django.urls import path

urlpatterns = [
    path("admin/", admin.site.urls),
    path("payments/", include("payments.urls")),
    path("test/", include("testapp.testmain.urls")),
]



import requests
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseForbidden

from .. import PaymentStatus
from .. import RedirectNeeded
from ..core import BasicProvider
from .forms import PaymentForm


class AuthorizeNetProvider(BasicProvider):
    """Payment provider for Authorize.Net.

    This backend implements payments using the Advanced Integration Method (AIM) from
    `Authorize.Net <https://www.authorize.net/>`_.

    This backend does not support fraud detection.

    :param login_id: Your API Login ID assigned by Authorize.net
    :param transaction_key: Your unique Transaction Key assigned by Authorize.net
    :param endpoint: The API endpoint to use. For the production environment, use
        ``'https://secure.authorize.net/gateway/transact.dll'`` instead.
    """

    def __init__(
        self,
        login_id,
        transaction_key,
        endpoint="https://test.authorize.net/gateway/transact.dll",
        **kwargs
    ):
        self.login_id = login_id
        self.transaction_key = transaction_key
        self.endpoint = endpoint
        super().__init__(**kwargs)
        if not self._capture:
            raise ImproperlyConfigured(
                "Authorize.Net does not support pre-authorization."
            )

    def get_transactions_data(self, payment):
        data = {
            "x_amount": payment.total,
            "x_currency_code": payment.currency,
            "x_description": payment.description,
            "x_first_name": payment.billing_first_name,
            "x_last_name": payment.billing_last_name,
            "x_address": "{}, {}".format(
                payment.billing_address_1, payment.billing_address_2
            ),
            "x_city": payment.billing_city,
            "x_zip": payment.billing_postcode,
            "x_country": payment.billing_country_area,
        }
        return data

    def get_product_data(self, payment, extra_data=None):
        data = self.get_transactions_data(payment)

        if extra_data:
            data.update(extra_data)

        data.update(
            {
                "x_login": self.login_id,
                "x_tran_key": self.transaction_key,
                "x_delim_data": True,
                "x_delim_char": "|",
                "x_method": "CC",
                "x_type": "AUTH_CAPTURE",
            }
        )

        return data

    def get_payment_response(self, payment, extra_data=None):
        post = self.get_product_data(payment, extra_data)
        return requests.post(self.endpoint, data=post)

    def get_form(self, payment, data=None):
        if payment.status == PaymentStatus.WAITING:
            payment.change_status(PaymentStatus.INPUT)
        form = PaymentForm(data=data, payment=payment, provider=self)
        if form.is_valid():
            raise RedirectNeeded(payment.get_success_url())
        return form

    def process_data(self, payment, request):
        return HttpResponseForbidden("FAILED")



from .. import PaymentStatus
from ..forms import CreditCardPaymentForm

RESPONSE_STATUS = {"1": PaymentStatus.CONFIRMED, "2": PaymentStatus.REJECTED}


class PaymentForm(CreditCardPaymentForm):
    def clean(self):
        cleaned_data = super().clean()

        if not self.errors:
            if not self.payment.transaction_id:
                data = {
                    "x_card_num": cleaned_data.get("number"),
                    "x_exp_date": cleaned_data.get("expiration"),
                    "x_card_code": cleaned_data.get("cvv2"),
                }
                response = self.provider.get_payment_response(self.payment, data)
                data = response.text.split("|")
                if response.ok and RESPONSE_STATUS.get(data[0], False):
                    status = RESPONSE_STATUS.get(data[0], PaymentStatus.ERROR)
                    self.payment.transaction_id = data[6]
                    if status == PaymentStatus.CONFIRMED:
                        self.payment.captured_amount = self.payment.total
                    self.payment.change_status(status)
                else:
                    errors = [data[3]]
                    self._errors["__all__"] = self.error_class(errors)
                    self.payment.change_status(PaymentStatus.ERROR)
        return cleaned_data



from django import forms

from payments import get_payment_model


class TestPaymentForm(forms.ModelForm):
    class Meta:
        model = get_payment_model()
        fields = ["variant", "currency", "total"]



from decimal import Decimal
from typing import Dict
from unittest import TestCase
from unittest.mock import MagicMock
from unittest.mock import Mock
from unittest.mock import patch

from django.core import signing

from .. import PaymentStatus
from .. import PurchasedItem
from .. import RedirectNeeded
from . import ACCEPTED
from . import AUTHENTICATE_REQUIRED
from . import TRANSACTION_SETTLED
from . import CyberSourceProvider

MERCHANT_ID = "abcd1234"
PASSWORD = "1234abdd1234abcd"
ORG_ID = "abc"

PROCESS_DATA = {
    "name": "John Doe",
    "number": "371449635398431",
    "expiration_0": "5",
    "expiration_1": "2023",
    "cvv2": "1234",
    "fingerprint": "abcd1234",
}


class Payment(Mock):
    id = 1
    variant = "cybersource"
    currency = "USD"
    total = 100
    status = PaymentStatus.WAITING
    transaction_id = None
    captured_amount = 0
    message = ""

    class attrs:
        fingerprint_session_id = "fake"
        merchant_defined_data: Dict[str, str] = {}

    def get_process_url(self):
        return "http://example.com"

    def get_failure_url(self):
        return "http://cancel.com"

    def get_success_url(self):
        return "http://success.com"

    def change_status(self, status, message=""):
        self.status = status
        self.message = message

    def get_purchased_items(self):
        return [
            PurchasedItem(
                name="foo",
                quantity=Decimal("10"),
                price=Decimal("20"),
                currency="USD",
                sku="bar",
            )
        ]


class TestCybersourceProvider(TestCase):
    @patch("payments.cybersource.suds.client.Client", new=MagicMock())
    def setUp(self):
        self.payment = Payment()
        self.provider = CyberSourceProvider(
            merchant_id=MERCHANT_ID, password=PASSWORD, org_id=ORG_ID
        )

    @patch.object(CyberSourceProvider, "_make_request")
    def test_provider_raises_redirect_needed_on_success(self, mocked_request):
        transaction_id = 1234
        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = 100
        mocked_request.return_value = response
        with self.assertRaises(RedirectNeeded):
            self.provider.get_form(payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(self.payment.status, PaymentStatus.CONFIRMED)
        self.assertEqual(self.payment.captured_amount, self.payment.total)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch.object(CyberSourceProvider, "_make_request")
    def test_provider_returns_form_on_3d_secure(self, mocked_request):
        response = MagicMock()
        response.reasonCode = AUTHENTICATE_REQUIRED
        mocked_request.return_value = response
        form = self.provider.get_form(payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)
        self.assertIn("PaReq", form.fields)

    @patch.object(CyberSourceProvider, "_make_request")
    def test_provider_shows_validation_error_message_response(self, mocked_request):
        error_message = (
            "The card you are trying to use was reported " "as lost or stolen."
        )
        error_code = 205
        response = MagicMock()
        response.reasonCode = error_code
        mocked_request.return_value = response
        form = self.provider.get_form(payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(form.errors["__all__"][0], error_message)

    def test_provider_shows_validation_error_message_duplicate(self):
        self.payment.transaction_id = 1
        error_message = "This payment has already been processed."
        form = self.provider.get_form(payment=self.payment, data=PROCESS_DATA)
        self.assertEqual(form.errors["__all__"][0], error_message)

    @patch.object(CyberSourceProvider, "_make_request")
    def test_provider_captures_payment(self, mocked_request):
        transaction_id = 1234
        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = TRANSACTION_SETTLED
        mocked_request.return_value = response
        self.provider.capture(self.payment)
        self.assertEqual(self.payment.status, PaymentStatus.CONFIRMED)

    @patch.object(CyberSourceProvider, "_make_request")
    def test_provider_refunds_payment(self, mocked_request):
        self.payment.captured_amount = self.payment.total
        response = MagicMock()
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response
        amount = self.provider.refund(self.payment)
        self.assertEqual(self.payment.total, amount)

    @patch.object(CyberSourceProvider, "_make_request")
    def test_provider_releases_payment(self, mocked_request):
        transaction_id = 123
        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response
        self.provider.release(self.payment)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch("payments.cybersource.redirect")
    @patch.object(CyberSourceProvider, "_make_request")
    def test_provider_redirects_on_success_captured_payment(
        self, mocked_request, mocked_redirect
    ):
        transaction_id = 1234
        xid = "abc"
        self.payment.attrs.xid = xid

        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response

        request = MagicMock()
        request.POST = {"MD": xid}
        request.GET = {
            "token": signing.dumps(
                {
                    "expiration": {"year": 2023, "month": 9},
                    "name": "John Doe",
                    "number": "371449635398431",
                    "cvv2": "123",
                }
            )
        }
        self.provider.process_data(self.payment, request)
        self.assertEqual(self.payment.status, PaymentStatus.CONFIRMED)
        self.assertEqual(self.payment.captured_amount, self.payment.total)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch("payments.cybersource.redirect")
    @patch.object(CyberSourceProvider, "_make_request")
    @patch("payments.cybersource.suds.client.Client", new=MagicMock())
    def test_provider_redirects_on_success_preauth_payment(
        self, mocked_request, mocked_redirect
    ):
        provider = CyberSourceProvider(
            merchant_id=MERCHANT_ID, password=PASSWORD, org_id=ORG_ID, capture=False
        )
        transaction_id = 1234
        xid = "abc"
        self.payment.attrs.xid = xid

        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = ACCEPTED
        mocked_request.return_value = response

        request = MagicMock()
        request.POST = {"MD": xid}
        request.GET = {
            "token": signing.dumps(
                {
                    "expiration": {"year": 2023, "month": 9},
                    "name": "John Doe",
                    "number": "371449635398431",
                    "cvv2": "123",
                }
            )
        }
        provider.process_data(self.payment, request)
        self.assertEqual(self.payment.status, PaymentStatus.PREAUTH)
        self.assertEqual(self.payment.captured_amount, 0)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    @patch("payments.cybersource.redirect")
    @patch.object(CyberSourceProvider, "_make_request")
    @patch("payments.cybersource.suds.client.Client", new=MagicMock())
    def test_provider_redirects_on_failure(self, mocked_request, mocked_redirect):
        transaction_id = 1234
        xid = "abc"
        self.payment.attrs.xid = xid

        response = MagicMock()
        response.requestID = transaction_id
        response.reasonCode = "test code"
        mocked_request.return_value = response

        request = MagicMock()
        request.POST = {"MD": xid}
        request.GET = {
            "token": signing.dumps(
                {
                    "expiration": {"year": 2023, "month": 9},
                    "name": "John Doe",
                    "number": "371449635398431",
                    "cvv2": "123",
                }
            )
        }
        self.provider.process_data(self.payment, request)
        self.assertEqual(self.payment.status, PaymentStatus.ERROR)
        self.assertEqual(self.payment.captured_amount, 0)
        self.assertEqual(self.payment.transaction_id, transaction_id)



from unittest import TestCase
from unittest.mock import MagicMock
from unittest.mock import Mock
from unittest.mock import patch

from .. import PaymentStatus
from .. import RedirectNeeded
from . import BraintreeProvider

MERCHANT_ID = "test11"
PUBLIC_KEY = "abcd1234"
PRIVATE_KEY = "1234abcd"

PROCESS_DATA = {
    "name": "John Doe",
    "number": "371449635398431",
    "expiration_0": "5",
    "expiration_1": "2023",
    "cvv2": "1234",
}


class Payment(Mock):
    id = 1
    variant = "braintree"
    currency = "USD"
    total = 100
    status = PaymentStatus.WAITING
    transaction_id = None
    captured_amount = 0

    def get_process_url(self):
        return "http://example.com"

    def get_failure_url(self):
        return "http://cancel.com"

    def get_success_url(self):
        return "http://success.com"

    def change_status(self, status):
        self.status = status


class TestBraintreeProvider(TestCase):
    def setUp(self):
        self.payment = Payment()

    def test_provider_redirects_to_success_on_payment_success(self):
        provider = BraintreeProvider(MERCHANT_ID, PUBLIC_KEY, PRIVATE_KEY)
        transaction_id = "12345"
        with patch("braintree.Transaction"):
            with patch("braintree.Transaction.sale") as mocked_sale:
                sale = MagicMock()
                sale.is_success = True
                sale.transaction.id = transaction_id
                mocked_sale.return_value = sale
                with self.assertRaises(RedirectNeeded) as exc:
                    provider.get_form(self.payment, data=PROCESS_DATA)
                    url = exc.args[0]
                    self.assertEqual(url, self.payment.get_success_url())
        self.assertEqual(self.payment.status, PaymentStatus.CONFIRMED)
        self.assertEqual(self.payment.captured_amount, self.payment.total)
        self.assertEqual(self.payment.transaction_id, transaction_id)

    def test_provider_shows_validation_error_message(self):
        provider = BraintreeProvider(MERCHANT_ID, PUBLIC_KEY, PRIVATE_KEY)
        error_msg = "error message"
        with patch("braintree.Transaction"):
            with patch("braintree.Transaction.sale") as mocked_sale:
                sale = MagicMock()
                sale.is_success = False
                sale.message = error_msg
                mocked_sale.return_value = sale
                form = provider.get_form(self.payment, data=PROCESS_DATA)
                self.assertEqual(form.errors["__all__"][0], error_msg)
        self.assertEqual(self.payment.status, PaymentStatus.ERROR)
        self.assertEqual(self.payment.captured_amount, 0)



from contextlib import contextmanager
from unittest import TestCase
from unittest.mock import Mock
from unittest.mock import patch

import stripe

from .. import FraudStatus
from .. import PaymentStatus
from .. import RedirectNeeded
from . import StripeCardProvider
from . import StripeProvider

SECRET_KEY = "1234abcd"
PUBLIC_KEY = "abcd1234"


class Payment(Mock):

    id = 1
    description = "payment"
    currency = "USD"
    delivery = 10
    status = PaymentStatus.WAITING
    message = None
    tax = 10
    total = 100
    captured_amount = 0
    transaction_id = None
    billing_email = "john@doe.com"

    def change_status(self, status, message=""):
        self.status = status
        self.message = message

    def change_fraud_status(self, status, message="", commit=True):
        self.fraud_status = status
        self.fraud_message = message

    def capture(self, amount=None):
        amount = amount or self.total
        self.captured_amount = amount
        self.change_status(PaymentStatus.CONFIRMED)

    def get_failure_url(self):
        return "http://cancel.com"

    def get_process_url(self):
        return "http://example.com"

    def get_purchased_items(self):
        return []

    def get_success_url(self):
        return "http://success.com"


@contextmanager
def mock_stripe_Charge_create(error_msg=None):
    json_body = {"error": {"charge": "charge_id"}}
    with patch("stripe.Charge.create") as mocked_charge_create:
        if error_msg:
            mocked_charge_create.side_effect = stripe.error.CardError(
                error_msg, param=None, code=None, json_body=json_body
            )
        else:
            mocked_charge_create.side_effect = lambda **kwargs: {}
        yield mocked_charge_create


@contextmanager
def mock_stripe_Charge_retrieve(fraudulent=False):
    with patch("stripe.Charge.retrieve") as mocked_charge_retrieve:
        fraud_details = {"stripe_report": None}
        if fraudulent:
            fraud_details["stripe_report"] = "fraudulent"
        mocked_charge_retrieve.side_effect = lambda charge_id: {
            "id": charge_id,
            "fraud_details": fraud_details,
        }
        yield mocked_charge_retrieve


class TestStripeProvider(TestCase):
    def test_form_contains_stripe_script(self):
        payment = Payment()
        store_name = "Test store"
        provider = StripeProvider(
            name=store_name, secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )
        form = provider.get_form(payment)
        self.assertTrue(
            '<script class="stripe-button" data-amount="10000" '
            'data-currency="USD" data-description="payment" data-email="john@doe.com" '
            'data-image="" data-key="%s" data-name="%s" '
            'src="https://checkout.stripe.com/checkout.js"></script>'
            % (PUBLIC_KEY, store_name)
            in str(form)
        )

    def test_form_contains_stripe_script_withou_billing_email(self):
        """
        If billing email is not set, it should generate the script as expected
        """
        payment = Payment()
        store_name = "Test store"
        provider = StripeProvider(
            name=store_name, secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )

        form = provider.get_form(payment)

        payment.billing_email = None
        form = provider.get_form(payment)
        self.assertTrue(
            '<script class="stripe-button" data-amount="10000" '
            'data-currency="USD" data-description="payment" '
            'data-image="" data-key="%s" data-name="%s" '
            'src="https://checkout.stripe.com/checkout.js"></script>'
            % (PUBLIC_KEY, store_name)
            in str(form)
        )

    def test_provider_raises_redirect_needed_when_token_does_not_exist(self):
        payment = Payment()
        provider = StripeProvider(
            name="Example.com store", secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )
        data = {}
        with self.assertRaises(RedirectNeeded) as exc:
            provider.get_form(payment, data)
            self.assertEqual(exc.args[0], payment.get_failure_url())
        self.assertEqual(payment.status, PaymentStatus.REJECTED)

    def test_provider_raises_redirect_needed_on_success(self):
        payment = Payment()
        provider = StripeProvider(
            name="Example.com store", secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )
        data = {"stripeToken": "abcd"}
        with patch("json.dumps"):
            with patch("stripe.Charge.create"):
                with self.assertRaises(RedirectNeeded) as exc:
                    provider.get_form(payment, data)
                    self.assertEqual(exc.args[0], payment.get_success_url())
        self.assertEqual(payment.status, PaymentStatus.CONFIRMED)
        self.assertEqual(payment.captured_amount, payment.total)

    def test_provider_shows_validation_error_message(self):
        error_msg = "Error message"

        payment = Payment()
        provider = StripeProvider(
            name="Example.com store", secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )
        data = {"stripeToken": "abcd"}
        with mock_stripe_Charge_create(error_msg=error_msg):
            with mock_stripe_Charge_retrieve():
                form = provider.get_form(payment, data=data)
                self.assertEqual(form.errors["__all__"][0], error_msg)
        self.assertEqual(payment.status, PaymentStatus.ERROR)
        self.assertEqual(payment.message, error_msg)
        self.assertEqual(payment.captured_amount, 0)

    def test_provider_set_fraudulent_status(self):
        error_msg = "Error message"
        payment = Payment()
        provider = StripeProvider(
            name="Example.com store", secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )
        data = {"stripeToken": "abcd"}
        with mock_stripe_Charge_create(error_msg=error_msg):
            with mock_stripe_Charge_retrieve(fraudulent=True):
                provider.get_form(payment, data=data)
        self.assertEqual(payment.status, PaymentStatus.ERROR)
        self.assertEqual(payment.fraud_status, FraudStatus.REJECT)
        self.assertEqual(payment.captured_amount, 0)

    def test_provider_detect_already_processed_payment(self):
        payment = Payment()
        payment.transaction_id = "existing_transaction_id"
        provider = StripeProvider(
            name="Example.com store", secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )
        data = {"stripeToken": "abcd"}
        with mock_stripe_Charge_create():
            with mock_stripe_Charge_retrieve():
                form = provider.get_form(payment, data=data)
                msg = "This payment has already been processed."
                self.assertEqual(form.errors["__all__"][0], msg)

    def test_form_doesnt_have_name_attributes_on_fields(self):
        payment = Payment()
        store_name = "Test store"
        provider = StripeCardProvider(
            name=store_name, secret_key=SECRET_KEY, public_key=PUBLIC_KEY
        )
        form = provider.get_form(payment)
        sensitive_fields = ["name", "cvv2", "expiration", "number"]
        for field_name in sensitive_fields:
            field = form[field_name]
            self.assertTrue("name=" not in str(field))



from decimal import Decimal

from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse
from django.http import HttpResponseForbidden

from ..core import BasicProvider
from .forms import ProcessPaymentForm

CENTS = Decimal("0.01")


class DotpayProvider(BasicProvider):
    """Payment provider for dotpay.pl

    This backend implements payments using a popular Polish gateway, `Dotpay.pl
    <http://www.dotpay.pl>`_.

    Due to API limitations there is no support for transferring purchased items.

    This backend does not support fraud detection.

    :param seller_id: Seller ID assigned by Dotpay
    :param pin: PIN assigned by Dotpay
    :param channel: Default payment channel (consult reference guide). Ignored if channel_groups is set.
    :param channel_groups: Payment channels to choose from (consult reference guide). Overrides channel.
    :param lang: UI language
    :param lock: Whether to disable channels other than the default selected above
    :param endpoint: The API endpoint to use. For the production environment, use ``'https://ssl.dotpay.pl/'`` instead
    :param ignore_last_payment_channel: Display default channel or channel groups instead of last used channel.
    :param type: Determines what should be displayed after payment is completed (consult reference guide).
    """

    _method = "post"

    def __init__(
        self,
        seller_id,
        pin,
        endpoint="https://ssl.dotpay.pl/test_payment/",
        channel=0,
        channel_groups=None,
        ignore_last_payment_channel=False,
        lang="pl",
        lock=False,
        type=2,
        **kwargs
    ):
        self.seller_id = seller_id
        self.pin = pin
        self.endpoint = endpoint
        self.channel = channel
        self.channel_groups = channel_groups
        self.ignore_last_payment_channel = ignore_last_payment_channel
        self.lang = lang
        self.lock = lock
        self.type = type
        super().__init__(**kwargs)
        if not self._capture:
            raise ImproperlyConfigured("Dotpay does not support pre-authorization.")

    def get_action(self, payment):
        return self.endpoint

    def get_hidden_fields(self, payment):
        if not payment.description:
            raise ValueError("Payment description is required")

        data = {
            "id": self.seller_id,
            "amount": Decimal(str(payment.total)).quantize(CENTS),
            "control": str(payment.id),
            "currency": payment.currency,
            "description": payment.description,
            "lang": self.lang,
            "ignore_last_payment_channel": "1"
            if self.ignore_last_payment_channel
            else "0",
            "ch_lock": "1" if self.lock else "0",
            "URL": payment.get_success_url(),
            "URLC": self.get_return_url(payment),
            "type": str(self.type),
        }
        if self.channel_groups:
            data["channel_groups"] = self.channel_groups
        else:
            data["channel"] = str(self.channel)
        return data

    def process_data(self, payment, request):
        form = ProcessPaymentForm(
            payment=payment, pin=self.pin, data=request.POST or None
        )
        if not form.is_valid():
            return HttpResponseForbidden("FAILED")
        form.save()
        return HttpResponse("OK")



from payments import PurchasedItem
from payments.models import BasePayment


class Payment(BasePayment):
    def get_failure_url(self):
        return "http://localhost:8000/test/payment-failure"

    def get_success_url(self):
        return "http://localhost:8000/test/payment-success"

    def get_purchased_items(self):
        yield PurchasedItem(
            name=self.description,
            sku="BSKV",
            quantity=1,
            price=self.total,
            currency=self.currency,
        )



import json

import requests
import xmltodict
from django.http import HttpResponseForbidden
from django.shortcuts import redirect
from django.template.loader import render_to_string
from django.utils.translation import get_language

from .. import PaymentError
from .. import PaymentStatus
from .. import RedirectNeeded
from ..core import BasicProvider


class SofortProvider(BasicProvider):
    """Payment provider for Sofort.

    This backend implements payments using `sofort.com <https://www.sofort.com/>`_ API.

    This backend does not support fraud detection.

    :param key: Your secret key
    :param id: Your sofort.com user id
    :param project_id: Your sofort.com project id
    :param endpoint: The API endpoint to use.
    """

    def __init__(
        self, key, id, project_id, endpoint="https://api.sofort.com/api/xml", **kwargs
    ):
        self.secret = key
        self.client_id = id
        self.project_id = project_id
        self.endpoint = endpoint
        super().__init__(**kwargs)

    def post_request(self, xml_request):
        response = requests.post(
            self.endpoint,
            data=xml_request.encode("utf-8"),
            headers={"Content-Type": "application/xml; charset=UTF-8"},
            auth=(self.client_id, self.secret),
        )
        doc = xmltodict.parse(response.content)
        return doc, response

    def get_form(self, payment, data=None):
        if not payment.id:
            payment.save()
        xml_request = render_to_string(
            "payments/sofort/new_transaction.xml",
            {
                "project_id": self.project_id,
                "language_code": get_language(),
                "interface_version": "django-payments",
                "amount": payment.total,
                "currency": payment.currency,
                "description": payment.description
                if len(payment.description) <= 40
                else (payment.description[:37] + "..."),
                "success_url": self.get_return_url(payment),
                "abort_url": self.get_return_url(payment),
                "customer_protection": "0",
            },
        )
        doc, response = self.post_request(xml_request)
        if response.status_code == 200:
            try:
                raise RedirectNeeded(doc["new_transaction"]["payment_url"])
            except KeyError:
                raise PaymentError(
                    "Error in {}: {}".format(
                        doc["errors"]["error"]["field"],
                        doc["errors"]["error"]["message"],
                    )
                )

    def process_data(self, payment, request):
        if "trans" not in request.GET:
            return HttpResponseForbidden("FAILED")
        transaction_id = request.GET.get("trans")
        payment.transaction_id = transaction_id
        transaction_request = render_to_string(
            "payments/sofort/transaction_request.xml",
            {"transactions": [transaction_id]},
        )
        doc, response = self.post_request(transaction_request)
        try:
            # If there is a transaction and status returned,
            # the payment was successful
            doc["transactions"]["transaction_details"]["status"]
        except KeyError:
            # Payment Failed
            payment.change_status(PaymentStatus.REJECTED)
            return redirect(payment.get_failure_url())
        else:
            payment.captured_amount = payment.total
            payment.change_status(PaymentStatus.CONFIRMED)
            payment.extra_data = json.dumps(doc)
            sender_data = doc["transactions"]["transaction_details"]["sender"]
            holder_data = sender_data["holder"]
            first_name, last_name = holder_data.rsplit(" ", 1)
            payment.billing_first_name = first_name
            payment.billing_last_name = last_name
            payment.billing_country_code = sender_data["country_code"]
            payment.save()
            return redirect(payment.get_success_url())

    def refund(self, payment, amount=None):
        if amount is None:
            amount = payment.captured_amount
        doc = json.loads(payment.extra_data)
        sender_data = doc["transactions"]["transaction_details"]["sender"]
        refund_request = render_to_string(
            "payments/sofort/refund_transaction.xml",
            {
                "holder": sender_data["holder"],
                "bic": sender_data["bic"],
                "iban": sender_data["iban"],
                "title": "Refund %s" % payment.description,
                "transaction_id": payment.transaction_id,
                "amount": amount,
                "comment": "User requested a refund",
            },
        )
        doc, response = self.post_request(refund_request)
        # save the response msg in "message" field
        # to start a online transaction one needs to upload the "pain"
        # data to his bank account
        payment.message = json.dumps(doc)
        payment.change_status(PaymentStatus.REFUNDED)
        return amount



import os

PROJECT_ROOT = os.path.normpath(os.path.join(os.path.dirname(__file__), "payments"))
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(PROJECT_ROOT, "templates")],
    }
]

SECRET_KEY = "NOTREALLY"
PAYMENT_HOST = "example.com"

INSTALLED_APPS = ["payments", "django.contrib.sites"]
